/******************************************************************************
 * g3DGMV
 * Copyright (C) <1999> <Nick Yost>
 *-----------------------------------------------------------------------------
 *  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 ******************************************************************************/

/* 3d vertex. */
typedef float vtexT[3];

/* 3d triangle made of 3 verticies. */
typedef struct triangle3dT{
  vtexT vtex[3];/* 3 xyz's. */
}triangle3dT;

/* 
  hline_endT is beginning or ending point of a horizontal "raster" line;
two arrays of these will define an envelope around the triangle.
*/
typedef struct hline_endT{
  float x,z;
}hline_endT;

/* Data structures used to define a map */
typedef struct row_list
    {
      gint16 gc;
      guint16  z;
     }row_list;

typedef struct col_list
    {
      guint16  lsize;
      guint16 row_offset;
      struct  row_list *ptr;
     }col_list;

typedef struct ElevMap
    {
      float NWXPOS;
      float NWYPOS;
      float SWXPOS; 
      float SWYPOS;
      float SEXPOS;
      float SEYPOS;
      float NEXPOS;
      float NEYPOS;
      gint GUnit; // defines the graphical format of the above 
      gint ElvUint;// define the graphicl format of elevation
      float minelev;
      float MaxElev;
      float spatial_resolution_x;// number of pixels per gunit in x
      float spatial_resolution_y;// number of pixels per gunit in y
      float spatial_resolution_z;// number of pixels per gunit in z
      struct col_list *col_list_ptr;
      GdkDrawable *pixmap;
      GdkPoint pixmap_size;
      GtkWidget *pbar;
      GdkImlibImage *background;
      gchar bg_str[250];
      gboolean bk_enable;
     }ElevMap;

		
/* --Global-- */
GdkGC *newgc;
GdkGC *invertgc;


/*-- Map house keeping routines --*/
ElevMap *gtk_create_Elevmap();

col_list *destroy_map(col_list *col_list_ptr );

/*--- Graphical content and color routines for map ---*/
GdkGC *GetPen(GdkDrawable *drawable, int c);

gint GetPenNumber(GdkGC *gc);

/*--- Drawing  routines for map ---*/
void gtk_Generate_Image(ElevMap *map,GdkDrawable *drawable,
			int ty,double xx,double yy,double zz);

void  map_draw_point    (col_list *demmap,gint gc,gint px, gint py); 

void Initilize_map(GdkDrawable *drawable);
 
void gtk_copy_gc_Map(col_list *demmap,GdkGC *gc,gint x,gint y);

void  gtk_map_draw_line    (ElevMap *map,GdkGC *gc1, gint start_x, 
			    gint start_y,gint end_x, gint end_y);

void  gtk_map_draw_lines    (ElevMap *demmap,GdkGC *gc1,
			     GdkPoint *points,gint numberoflines );

void  map_draw_line    (col_list *demmap, gint gc, gint start_x, gint start_y,
			gint end_x, gint end_y);

void  gtk_map_draw_point    (ElevMap *map,GdkGC *gc1,gint px, gint py);

void  gtk_map_draw_rectangle (ElevMap *demmap,GdkGC *gc1,gboolean fill,
			      gint start_x, gint start_y, gint rec_width, 
			      gint rec_height);

void  map_filler    (col_list *demmap,gint gc,gint px, gint py,
		     gint fillbox_top,gint fillbox_bottom,
		     gint fillbox_left,gint fillbox_right) ;

void  gtk_map_fill_bounded    (ElevMap *map,GdkGC *gc,gint px, gint py); 

void  gtk_map_fill_replace (ElevMap *map,GdkGC *gc,gint px, gint py);

void  map_set_color_range(ElevMap *map,gint range);
gint  map_get_water_level(ElevMap *map);
void  map_set_water_level(ElevMap *map,gint water);

gint map_get_elv(col_list *demmap, gint px,gint py);

void  gtk_map_draw_face  (ElevMap *map,GdkGC *gc1,
			  float x0,float y0,float z0,
			  float x1,float y1,float z1, 
			  float x2,float y2,float z2, 
			  float x3,float y3,float z3);
GdkPoint Resize_pixmap(ElevMap *map,GdkDrawable *drawable,gboolean normal);

void Generate_Wire_frame(ElevMap *map, GdkDrawable *drawable,
			 int LeftOffset, int TopOffset, int Width, int Height,int Step,
			 double xx,double yy,double zz,gboolean FrameMode, 
			 gboolean MapColor, float zoom);
//-------------------------------------------------------------------
// Map Helper Macros
//-------------------------------------------------------------------
#define MapElev(Map,col,row) (Map->col_list_ptr[col].ptr[row].z)

#define MapColor(Map,col,row) (Map->col_list_ptr[col].ptr[row].gc)

#define MapRowOffset(Map,col) (Map->col_list_ptr[col].row_offset)

#define MapRowPos(Map,col,row) (MapRowOffset(Map,col)-row)
